# NODE BNGM benchmark analysis

## Aim

This repository contains all scripts and data necessary to perform the benchmark analysis described in the manuscript.
The purpose of the benchmark analysis is to compare the runtime and accuracy of neural ordinary differential equations (NODEs) fitted by Bayesian neural gradient matching (BNGM) to standard NODEs, parametric ODEs with second order polynomial, and convergent cross mapping (CCM).


## Inputs

The analysis consisted in fitting an `artificial time series` of a tri-trophic system with a prey, intermediate predator, and top predator.
The artificial time series are generated by a Lotka-Volerra ODE system with known parameters.
This means that ground truth for effects and contributions of each variable to the dynamics of the others is known.
The methods also requires user-specified `hyperparameters`, such as the number of ensemble samples, the size of the networks, etc.


## Outputs

Each method provides the following quantities: 
* the `per-capita growth rate` of each species as a function of the density of the other species
* the `effects` of each species on the dynamics of another
* the `contributions` of each species to the dynamics of another
* the `runtimes` in seconds required to execute the methods

These outputs are combined into benchmark figures that compare the accuracies and runtimes of the four approaches.


## Installation

All necessary scripts are present in the respective folders and require R (v4.0.2 or later).
The CCM method requires the external library rEDM, which can be installed by running

``` R
install.packages("rEDM")
```


## Running scripts

### Generating the artificial time series

Generating the artificial time series can be done within the `GT` (ground truth) repository.
This can be achieved by running the script `m_3DLV.r`.
This script creates the time series in csv format `out/TS_3DLV.csv`.

### Running NODE via BNGM

Fitting NODEs to the times series with the BNGM approach can be done within the repository `NODEBNGM`.

This can be done by running the script `m0_main_v0_2.r`, which takes as input the artificial time series `data/TS_3DLV.csv`.

This script produces the following files:
* `NODEBNGM/out/Yhat_p.RData` the approximation of the per-capita growth rate as a function of the interpolated state variables
* `NODEBNGM/out/ddx.Yhat_p.RData` the sensitivity of the per-capita growth rate with respect to the interpolated state variables
* `NODEBNGM/out/Geber_p.RData` the contributions of each state variable to changes in the per-capita growth rate

### Running standard NODEs

Fitting standard NODEs to the time series can be done within the `NODE` repository.

This can be done by running the script `m_NODE.r`, which takes as input the artificial time series `data/TS_3DLV.csv`. 

This script generates the RData object `out/results_NODE.RData`, which contains the following quantities:
* `Yhat_p.RData` the approximation of the per-capita growth rate as a function of the interpolated state variables
* `ddx.Yhat_p.RData` the sensitivity of the per-capita growth rate with respect to the interpolated state variables
* `Geber_p.RData` the contributions of each state variable to changes in the per-capita growth rate

### Running parametric ODEs

Fitting parametric ODEs to the time series can be done within the `ODE2` repository.

This can be done by running the script `m_ODE2.r`, which takes as input the artificial time series `data/TS_3DLV.csv`. 

This script generates the RData object `out/results_ODE2.RData`, which contains the following quantities:
* `Yhat_p.RData` the approximation of the per-capita growth rate as a function of the interpolated state variables
* `ddx.Yhat_p.RData` the sensitivity of the per-capita growth rate with respect to the interpolated state variables
* `Geber_p.RData` the contributions of each state variable to changes in the per-capita growth rate

### Running CCM

Analysing the time series with CCM can be done within the `CCM` repository.

This can be done by running the script `m_CCM.r`, which takes as input the artificial time series `data/TS_3DLV.csv`.

This script generates the RData object `out/results_CCM.RData`, which contains the following quantities:
* `Yhat_p.RData` the approximation of the per-capita growth rate as a function of the interpolated state variables
* `ddx.Yhat_p.RData` the sensitivity of the per-capita growth rate with respect to the interpolated state variables
* `Geber_p.RData` the contributions of each state variable to changes in the per-capita growth rate

### Creating benchmark figures 

Benchmark figures used in the manuscript can be generated by running the script `m_benchmark.r`.

This script aggregates the results of the 4 methods (NODEBNGM, NODEs, ODE2, and CCM).

It produces three benchmark figures:
* `out/benchmark_overview.pdf` shows a comparison of the overall accuracy and runtimes of the four approaches
* `out/benchmark_test.pdf` shows a detailed comparison of the accuracy of the four approaches in predicting effects and contributions of the variables in the test set (unseen data)
* `out/benchmark_test.pdf` shows a detailed comparison of the accuracy of the four approaches in predicting effects and contributions of the variables in the training set (seen data)
